/*
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#include "update_funds.h"

struct fund
	{
	char *name;
	float close_price;
	long volume;	
	int year;
	int month;
	int monthday;
	struct fund *nxtentr;
	struct fund *prventr;
	};
struct fund *fundtab[2];


struct fund *lookup_fund(char *name)
{
struct fund *pa;

if(debug_flag) 
	{
	fprintf(stdout, "lookup_fund(): arg name=%s\n", name);
	}

for(pa = fundtab[0]; pa != 0; pa = pa -> nxtentr)
	{
	if(strcmp(pa -> name, name) == 0) return(pa);
	}
return(0); /* not found */
}


struct fund *install_fund_at_end_of_list(char *name)
{
struct fund *plast, *pnew;
struct fund *lookup_fund();

if(debug_flag)
	{
	fprintf(stdout, "install_fund_at_end_of_list(): arg name=%s\n", name);
	}

pnew = lookup_fund(name);
if(pnew)
	{
	/* else already there */
	if(debug_flag)
		{
		fprintf(stdout,\
		"install_fund_at_end_of_list(): already there ignored\n");
		}
	return(pnew);/* allow double entries in file, return old address */
	}

/* create new structure */
pnew = (struct fund *) calloc(1, sizeof(*pnew) );
if(! pnew) return(0);
pnew -> name = strsave(name);
if(! pnew -> name) return(0);

/* get previous structure */
plast = fundtab[1]; /* end list */

/* set new structure pointers */
pnew -> nxtentr = 0; /* new points top zero (is end) */
pnew -> prventr = plast; /* point to previous entry, or 0 if first entry */

/* set previuos structure pointers */
if( !fundtab[0] ) fundtab[0] = pnew; /* first element in list */
else plast -> nxtentr = pnew;

/* set array end pointer */
fundtab[1] = pnew;

return(pnew);/* pointer to new structure */
}/* end function install_fund_at_end_of_list */


int delete_fund(char *name)/* delete entry from double linked list */
{
struct fund *pa, *pprev, *pdel, *pnext;

if(debug_flag)
	{
	fprintf(stdout, "delete_fund(): arg name=%s\n", name);
	}

/* parameter check */
if(! name) return(0);

pa = fundtab[0];
while(1)
	{
	/* if end list, return not found */
	if(! pa) return(0);

	/* test for match in name */
	if(strcmp(name, pa -> name) != 0) /* no match */
		{
		/* point to next element in list */
		pa = pa -> nxtentr;

		/* loop for next element in list */
		continue;
		}

	/* we now know which struture to delete */
	pdel = pa;

	/* get previous and next structure */
	pnext = pa -> nxtentr;
	pprev = pa -> prventr;

	/* set pointers for previous structure */
	/* if first one, modify fundtab[0] */
	if(pprev == 0) fundtab[0] = pnext;
	else pprev -> nxtentr = pnext;

	/* set pointers for next structure */
	/* if last one, modify fundtab[1] */
	if(pnext == 0) fundtab[1] = pprev;
	else pnext -> prventr = pprev;
	
	/* delete structure */	
	/* delete structure data */	
	free(pdel -> name);/* free name */
	free(pdel); /* free structure */

	/* return OK deleted */
	return(1);
	}/* end for all structures */
}/* end function delete_fund */


int delete_all_funds()/* delete all entries from table */
{
struct fund *pa;

if(debug_flag)
	{
	fprintf(stdout, "delete_all_funds(): arg none\n");
	}

while(1)
	{
	pa = fundtab[0];
	if(! pa) break;
	fundtab[0] = pa -> nxtentr;/* fundtab entry points to next one,
												            this could be 0 */
	free(pa -> name);/* free name */

	free(pa);/* free structure */
	}/* end for all structures */
fundtab[1] = 0;
return(0);/* not found */
}/* end function delete_all_funds */


int get_fund_from_database(\
char *fundname, float *close_price, long *volume,\
int *monthday, int *month, int *year)
{
int c, i;
struct fund *pa;
char fundname_uppercase[READSIZE];

i = 0;
while(1)
	{
	c = fundname[i];
	fundname_uppercase[i] = toupper(c);
	if(c == 0) break;
	i++;
	}

pa = lookup_fund(fundname_uppercase);
if(! pa) return(0);

*close_price = pa -> close_price;
*volume = pa -> volume;
*monthday = pa -> monthday;
*month = pa -> month;
*year = pa -> year;

return(1);
}/* end function get_fund_from_database */


int add_fund_to_database(\
char *fundname, float closeprice, long volume,\
int monthday, int month, int year)
{
struct fund *pa;

if(debug_flag)
	{
	fprintf(stdout,\
	"add_fund_to_database(): arg fundname=%s closeprice=%.2f volume=%ld\n\
	monthday=%d month=%d year=%d\n",\
	fundname, closeprice, volume,\
	monthday, month, year);
	}

if(! fundname) return(0);

pa = install_fund_at_end_of_list(fundname);
if(! pa)
	{
	fprintf(stdout,\
	"add_fund_to_database(): cannot install fund %s\n", fundname);	
	return(0);
	}

pa -> name = strsave(fundname);
if(! pa -> name)
	{
	fprintf(stdout, "add_fund_to_database(): malloc failed\n");
	return(0);
	}

pa -> close_price = closeprice;
pa -> volume = volume;
pa -> monthday = monthday;
pa -> month = month;
pa -> year = year;

return(1);
}/* end function add_fund_to_database */


