/*
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#include "update_funds.h"


int parse_iris_htm_file(char *filename)
{
int a, c, i;
struct stat *statptr;
char *space, *spaceptr;
FILE *fileptr;
size_t size;
char *ptr1, *ptr2, *ptr3, *ptr4;
int symbolflag, lastflag, volflag, timeflag, dateflag;
char symbolstr[512];
char laststr[512];
char volstr[512];
char timestr[512];
char datestr[512];
int symboloffset, lastoffset, voloffset, timeoffset, dateoffset;
int have_format_flag;
float closeprice;
long volume;
char monthdaystr[20];
char monthstr[20];
char yearstr[20];
int monthday, month, year;

if(debug_flag)
	{
	fprintf(stdout, "parse_iris_htm_file arg filename=%s\n", filename);
	}

if(! filename) return(0);

/* for gcc -Wall */
symboloffset = 0;
lastoffset = 0;
voloffset = 0;
timeoffset = 0;
dateoffset = 0;

fileptr = fopen(filename, "r");
if(! fileptr)
	{
	if(debug_flag)
		{
		fprintf(stdout,\
		"parse_iris_htm_file(): cannot open file %s for read\n",\
		filename);
		}
	return(0);
	}/* end if open fails */
/*fprintf(stdout, "file open\n");*/

statptr = (struct stat*) malloc(sizeof(struct stat) );
if(! statptr) return(0);

fstat(fileno(fileptr), statptr);
size = statptr -> st_size;
if(size == 0)
	{
	free(statptr);
	return(0);
	}

space = malloc(statptr -> st_size + 1);
free(statptr);
if(! space)
	{
	if(debug_flag)
		{
		fprintf(stdout,\
		"parse_iris_htm_file():  malloc failed\n");
		}
	return(0);
	}

spaceptr = space;
while(1)
	{
	while(1)/* error re read */
		{
		c = getc(fileptr);
		if(! ferror(fileptr) ) break;
		perror(" parse_iris_htm_file():  read failed ");
		}/* end while error re read */	

	if(feof(fileptr) )
		{
		fclose(fileptr);
		break;
		}
	*spaceptr = c;
	spaceptr++;
	}/* end while all lines from article body */
/* may not be there */
*spaceptr = 0;/* string termination */

/*fprintf(stdout, "space=%s\n", space);*/

have_format_flag = 0;
ptr2 = space;
while(1)
	{
	ptr1 = strcasestr(ptr2, "<BR>"); 
	if(ptr1 != ptr2)
		{
		ptr2++;
		if(ptr2 > (space + size) ) break;
		continue;
		}

	/* point to fund name (hopefully) */
	ptr1 += 4;

	/* clear these */
	symbolstr[0] = 0;
	laststr[0] = 0;
	volstr[0] = 0;
	timestr[0] = 0;
	datestr[0] = 0;

/*fprintf(stdout, "fundname=%s ptr2=%lu\n", fundname, (long)ptr2);*/

/* Symbol Last NetCh %Ch MBid MAsk BS AS Vol Time Date Prev Open High Low */
		
	/*
	Some arguments on a line may be missing.
	Using the above description field line as reference.
	The data is not left, but right justified relative to these fields,
	so have to use sscanf from the end of the preceeding field.
	Man what a mess.
	Symbol however is right justified.
	*/
	
	/* get the fundname */
	a  = sscanf(ptr1, "%s", symbolstr);
	if(a == 1)
		{
		if(strcmp(symbolstr, "Symbol") == 0)
			{
			/* pointers to end of fields */
			symboloffset = 0;
			lastoffset = strstr(ptr1, "Last") - ptr1 + 4;
			voloffset = strstr(ptr1, "Vol") - ptr1 + 3;
			timeoffset = strstr(ptr1, "Time") - ptr1 + 4;
			dateoffset = strstr(ptr1, "Date") - ptr1 + 4;
			have_format_flag = 1;
		
			ptr2 += 4;
			if(ptr2 > (space + size) ) break;
			continue;
			}/* end if Symbol */
		}/* end if a == 1 */
	if(! have_format_flag)
		{
		ptr2 += 4;
		if(ptr2 > (space + size) ) break;
		continue;
		}

	/* test if this is a valid fundname */
	symbolflag = 1;
	for(i = 0; i < strlen(symbolstr); i++)
		{
		c = symbolstr[i];
			
		/* digits allowed */
		if(isdigit(c) ) continue;
			
		/* only A to Z uppercase allowed */
		if( (c < 'A') || (c > 'Z') )
			{
			symbolflag = 0;
			break;
			}
		}/* end for all chars in symbolstr */

	if(! symbolflag)
		{
		ptr2 += 4;
		if(ptr2 > (space + size) ) break;
		continue;
		}
/*fprintf(stdout, "symbolstr=%s\n", symbolstr);*/

	/* Get Last */
	/* move back from lastoffset until whitespace */
	lastflag = 0;
	ptr3 = ptr1 + lastoffset - 1;
	while(1)
		{
		c = *ptr3;
		if( isspace(c) ) break;
		else lastflag = 1;
		ptr3--;
		if(ptr3 <= (ptr1 + symboloffset) ) break;
		}
	
	/* if Last field not present next fund */
	if(! lastflag)
		{
		ptr2 += 4;
		if(ptr2 > (space + size) ) break;
		continue;
		}
	
	/* copy 'last' field to laststr */ 
	ptr3++;
	ptr4 = laststr; 
	while(1)
		{
		c = *ptr3;
		if( isspace(c) )
			{
			*ptr4 = 0;
			break;
			}			

		*ptr4 = c;
		ptr3++;
		ptr4++;
		}
/*fprintf(stdout, "laststr=%s\n", laststr);*/

	/* get Vol */
	/* move back from voloffset until whitespace */
	volflag = 0;
	ptr3 = ptr1 + voloffset - 1;
	while(1)
		{
		c = *ptr3;
		if( isspace(c) ) break;
		else volflag = 1;
		ptr3--;
		if(ptr3 <= (ptr1 + lastoffset) ) break;
		}
	
	if(! volflag)
		{
		ptr2 += 4;
		if(ptr2 > (space + size) ) break;
		continue;
		}
	
	/* copy vol field to volstr */
	ptr3++;
	ptr4 = volstr; 
	while(1)
		{
		c = *ptr3;
		if( isspace(c) )
			{
			*ptr4 = 0;
			break;
			}			

		*ptr4 = c;
		ptr3++;
		ptr4++;
		}
/*fprintf(stdout, "volstr=%s\n", volstr);*/

	/* get Time */
	/* move back from timeoffset until whitespace */
	timeflag = 0;
	ptr3 = ptr1 + timeoffset - 1;
	while(1)
		{
		c = *ptr3;
		if( isspace(c) ) break;
		else timeflag = 1;
		ptr3--;
		if(ptr3 <= (ptr1 + voloffset) ) break;
		}
	
	if(! timeflag)
		{
		ptr2 += 4;
		if(ptr2 > (space + size) ) break;
		continue;
		}
	
	/* copy date field to datestr */
	ptr3++;
	ptr4 = timestr; 
	while(1)
		{
		c = *ptr3;
		if( isspace(c) )
			{
			*ptr4 = 0;
			break;
			}			

		*ptr4 = c;
		ptr3++;
		ptr4++;
		}
/*fprintf(stdout, "timestr=%s\n", timestr);*/

	/* get Date */
	/* move back from dateoffset until whitespace */
	dateflag = 0;
	ptr3 = ptr1 + dateoffset - 1;
	while(1)
		{
		c = *ptr3;
		if( isspace(c) ) break;
		else dateflag = 1;
		ptr3--;
		if(ptr3 <= (ptr1 + timeoffset) ) break;
		}
	
	if(! dateflag)
		{
		ptr2 += 4;
		if(ptr2 > (space + size) ) break;
		continue;
		}
	
	/* copy date field to datestr */
	ptr3++;
	ptr4 = datestr; 
	while(1)
		{
		c = *ptr3;
		if( isspace(c) )
			{
			*ptr4 = 0;
			break;
			}			

		*ptr4 = c;
		ptr3++;
		ptr4++;
		}
/*fprintf(stdout, "datestr=%s\n", datestr);*/

	if(debug_flag)
		{
		fprintf(stdout,\
		"symbolstr=%s laststr=%s volstr=%s timestr=%s datestr=%s\n",\
		symbolstr, laststr, volstr, timestr, datestr); 
		}

	closeprice = atof(laststr);
	volume = atol(volstr);
	
	/* 19981102  november */
	sscanf(datestr, "%4s%2s%2s", yearstr, monthstr, monthdaystr);
	monthday = atoi(monthdaystr);
	month = atoi(monthstr);
	year = atoi(yearstr);	

	if(! add_fund_to_database(\
	symbolstr, closeprice, volume, monthday, month, year) )
		{
		fprintf(stdout,\
		"parse_iris_htm_file(): cannot add fund to database.\n");
		return(0);
		}
	
	/* scip <BR> */
	ptr2 += 4;
	if(ptr2 > (space + size) ) break;
	}/* end while all lines */

free(space);
return(1);
}/* end function parse iris_htm_file */


int parse_quoteweb_csv_file(char *filename)
{
FILE *fileptr;
char temp[READSIZE];
char *ptr;
int a;
char fundname[512];
char datestr[80];
float closeprice;
long volume;
int monthday, year, month;

if(debug_flag)
	{
	fprintf(stdout,\
	"parse_quoteweb_csv_file(): arg filename=%s\n",\
	filename);
	}

if(! filename) return(0);

fileptr = fopen(filename, "r");
if(! fileptr)
	{
	fprintf(stdout,\
	"parse_quoteweb_csv_file(): cannot open file %s for read\n",\
	filename);
	return(0);
	}

while(1)
	{
	/* ABN AMRO HOLDING,AAB,06/11/1998,40.1,40.3,38.8,38.8,8774554
 */
	a = readline(fileptr, temp);/* closes file on EOF */
	if(a == EOF) break;

	ptr = strtok(temp, ",");/* description  ABN AMRO HOLDING */
	if(! ptr) continue;

	ptr = strtok(NULL, ",");/* abbreviation AAB */
	if(! ptr) continue;
	strcpy(fundname, ptr);

	ptr = strtok(NULL, ",");/* date 06/11/1998 */
	if(! ptr) continue;
	strcpy(datestr, ptr);	

	ptr = strtok(NULL, ",");/* ?? 40.1 */
	if(! ptr) continue;
		
	ptr = strtok(NULL, ",");/* ?? ,40.3 */
	if(! ptr) continue;

	ptr = strtok(NULL, ",");/* ?? 38.8 */
	if(! ptr) continue;

	ptr = strtok(NULL, ",");/* close price 38.8 */
	if(! ptr) continue;
	closeprice = atof(ptr);
		
	ptr = strtok(NULL, ",");/* volume 8774554 */
	if(! ptr) continue;
	volume = atof(ptr);

	if(debug_flag)
		{
		fprintf(stdout,\
		"fund=%s date=%s close=%.2f volume=%ld\n",\
		fundname, datestr, closeprice, volume);
		}

	/* datestr now looks like 06/11/1998 */
	sscanf(datestr, "%d/%d/%d", &monthday, &month, &year);

	if(debug_flag)
		{
		fprintf(stdout,\
		"monthday=%d year=%d month=%d\n",\
		monthday, year, month);
		}

	if(! add_fund_to_database(\
	fundname, closeprice, volume, monthday, month, year) )
		{
		return(0);
		}

	}/* end while all lines in .csv file */

return(1);
}/* end function parse_quoteweb_csv_file */

