/*
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#include "update_funds.h"

int mdays[13] = {0,31,28,31,30,31,30,31,31,30,31,30,31};
char *month_name[]=\
{"illegal","jan","feb","mrt","apr","mei","jun","jul",\
"aug","sep","okt","nov","dec"};
char *day[] = {"ma","di","wo","do","vr","za","zo"};
char *day_name[] = {"zo", "ma", "di", "wo", "do", "vr", "za"};


int update_fund_file(\
char *fundname, float close_price, long volume,\
int monthday, int month, int year)
{
int volume_flag;

if(debug_flag)
	{
	fprintf(stdout,\
	"update_fund_file():\n\
	arg fundname=%s close_price=%.2f volume=%ld\n\
	monthday=%d month=%d year=%d\n",\
	fundname, close_price, volume,\
	monthday, month, year);
	}

if(! fundname) return(0);

volume_flag = 1;

if(! update_krs_file(\
fundname, close_price, volume, year, month, monthday, NEW_DATA) )
	{
	return(0);
	}

if(! update_csv_csw(\
	fundname, year, month, monthday, close_price, volume, volume_flag) )
	{
	fprintf(stdout, "update_fund_file(): cannot update fund %s\n", fundname);
	return(0);
	}	

return(1);
}/* end function update_fund_file */


int update_csv_csw(\
char *filebody, int year, int month, int mtday,\
float closeprice, long volume,\
int volume_flag)
{
/*test if file exist*/
char stock[40];
char path[READSIZE];
FILE *csvcswfile;

if(debug_flag)
	{
	fprintf(stdout,\
	"update_csv_csw(): arg filebody=%s year=%d month=%d mtday=%d\n\
	closeprice=%.2f volume=%ld volume_flag=%d\n",\
	filebody, year, month, mtday, closeprice, volume, volume_flag);
	}

get_path_name("stock_files_dir", path);

/* csv always */
sprintf(stock, "%s%s.csv", path, filebody);

/*test if file exists*/
csvcswfile = fopen(stock,"r");
if(csvcswfile)
	{
	/* exists, not going to read, but append, using "a" would create new. */
	fclose(csvcswfile);
	}
else /*not existing, try to create a new file*/
	{
	fprintf(stdout, "Cannot find file %s, creating it.\n", stock);
   	csvcswfile = fopen(stock, "w");
  	if(!csvcswfile)/*no create*/
  		{
  		fprintf(stdout, "\nCannot create file %s either.\n", stock);
  		return(0);
  		}
	/*
	created ok
	print the header, D stands for date (see following for format),
	C stands for closing price, V stands for volume.
	*/

	fprintf(csvcswfile, "DC\n");/*print header*/

	fclose(csvcswfile);/*close*/
	}/* end if not existing */

/* now we have for sure a csv file */
/* append new data */
csvcswfile = fopen(stock, "a");/*open for append*/
if(!csvcswfile)
	{
	fprintf(stdout,\
	"update_csv_csw(): cannot open file %s.csv for append\n", stock);
	return(0);
	}

/* .csv file now open for append, update it */
/* the format is mm/dd/yy,closeprice */
fprintf(csvcswfile,\
"%02d/%02d/%02d,%4.2f\n",\
month, mtday, year % 100, closeprice);

fclose(csvcswfile);

/* csw only if volme */
if(! volume_flag) return(1);
	
sprintf(stock, "%s%s.csw", path, filebody);

/*test if file exists*/
csvcswfile = fopen(stock,"r");
if(csvcswfile)
	{
	/* exists, not going to read, but append, using "a" would create new. */
	fclose(csvcswfile); 
	}
else /*not existing, try to create a new file*/
	{
	fprintf(stdout, "Cannot find file %s, creating it.\n", stock);
	csvcswfile = fopen(stock, "w");
	if(! csvcswfile)/*no create*/
		{
		fprintf(stdout, "Cannot create file %s either.\n", stock);
		return(0);
		}
	/*
	created ok
	print the header, D stands for date (see following for format),
	C stands for closing price, V stands for volume.
	*/

	fprintf(csvcswfile, "DCV\n");/*print header*/

	fclose(csvcswfile);/*close*/
	}/* end if not existing */

/* now we have for sure a .csw file*/
/* append new data */
csvcswfile = fopen(stock, "a");/*open for append*/
if(! csvcswfile)
	{
	fprintf(stdout,\
	"update_csv_csw(): cannot open file %s.csw for append\n", stock);
	return(0);
	}

/* .csw file now open for append, update it */
/* the format is mm/dd/yy,closeprice */
fprintf(csvcswfile,\
"%02d/%02d/%02d,%4.2f,%lu\n",\
month, mtday, year % 100, closeprice, volume);
	
fclose(csvcswfile);

return(1);
}/* end function update_csv_csw */


int update_krs_file(\
char *fundname, float close_price, long volume,\
int htm_file_year, int htm_file_month, int htm_file_day_of_month,\
int data_type)
{
int a, i;
float fa, fb;
char fiwd[10];/*weekday in header stockfile*/
char fimo[10];/*month in header stockfile*/
int fida;/*day in header stockfile*/
char path[READSIZE];
char ti[READSIZE];
char to[READSIZE];
char ta[READSIZE];
float fmaxk,fmink;
int xpos,ypos,lenfie,lenfr;
float trend;
int pg,sp,af;/*page subpage acq flag*/
FILE *krsf;
FILE *krsfb;
char si[80];/*scanf input string*/
char comstr[21];/*comment string for .krs file*/
char oldmonth[12];
int oldyear;
char temp[READSIZE];
int xcnt, xflag, xdeflag;
int special_field_appended_flag;
struct tm *htm_file_time;
struct tm *krs_file_time;
time_t new_data_time;
time_t old_data_time;
int krs_file_day_of_month;
int krs_file_month;
int krs_file_year;

if(debug_flag)
	{
	fprintf(stdout,\
	"update_krs_file(): arg fundname=%s close_price=%.2f volume=%ld\n\
	htm_file_year=%d htm_file_month=%d htm_file_day_of_month=%d\n\
	data_type=%d\n",\
	fundname, close_price, volume,\
	htm_file_year, htm_file_month, htm_file_day_of_month,\
	data_type);
	}

if(! fundname) return(0);

/* for gcc -Wall */
fb = 0;

/* calculate time_t new_data_time */
htm_file_time = (struct tm *) malloc(sizeof(struct tm) );
if(! htm_file_time) return(0);

htm_file_time -> tm_sec = 0;
htm_file_time -> tm_min = 0;
htm_file_time -> tm_hour = 0;
htm_file_time -> tm_mday = htm_file_day_of_month;
htm_file_time -> tm_mon = htm_file_month - 1;
htm_file_time -> tm_year = htm_file_year - 1900;
htm_file_time -> tm_isdst = -1;/* daylight savings time n.a. */
/* the following are not implemented (see /usr/include/time.h) */
/* htm_file_time -> tm_gmtoff = 0;*/
/* strcp(htm_file_time -> tm_zone, ""); */ /* time zone 3 letters */

new_data_time = mktime( (struct tm *)htm_file_time);

free(htm_file_time);

get_path_name("stock_files_dir", path);
sprintf(ti, "%s%s.krs", path, fundname);/* source */	
sprintf(to, "%s%s.tmp", path, fundname);/* to is fundname.tmp */	
sprintf(ta, "%s%s.bak", path, fundname);/* ta will be ti renamed in .bak */	

/*open files*/
krsf=fopen(to,"w");/*krsf is fundname.tmp, dest*/
if(! krsf)
	{
	sprintf(temp,\
	"update_krs_file(): cannot open file %s for write\n", to);
	updateerrorfile(temp);
	fprintf(stdout,"%s", temp);
	beep(2,3000);
	return(0);
	}

krsfb = fopen(ti,"r");/*krsfb is fundname.krs, source*/
if(! krsfb)
	{
	sprintf(temp,\
	"update_krs_file(): cannot open file %s for read\n", ti);
	updateerrorfile(temp);

	fclose(krsf);/*temp was open*/

	fprintf(stdout, "%s", temp);
	beep(1,3000);
	return(0);
	}
			
fscanf(krsfb,\
"%s %s %d %s %f %d %d %d %d %d %d %f %f %d",\
fiwd,fimo,&fida,si,&trend,&af,&pg,&sp,&xpos,&ypos,&lenfie,\
&fmaxk,&fmink,&lenfr);/* get header from old file */

/* calculate time_t old_data_time (time last updated) */
/* 
Problem is that we do not have the year here from the file,
unles we read it first (last year).
Solution, from now on use fiwd (the day of week in Dutch) as year.
One could put old_data_time here, but not human readable.
So now 1998 okt 30 aab 1.00 etc. .
*/

krs_file_day_of_month = fida;
krs_file_month = -1;
for(i = 0; i <= 12; i++)/* 0 is illegal, 12 is dec */
	{
	if(strcmp(fimo, month_name[i]) == 0)
		{
		krs_file_month = i;
		break;
		}
	}
if(krs_file_month == -1)	
	{
	sprintf(temp,\
	"update_krs_file(): file %s invalid month %s\n",\
	ti, fimo);
	updateerrorfile(temp);
	fprintf(stdout, "%s", temp);
	fclose(krsf);
	fclose(krsfb);
	beep(2,3000);
	return(0);
	}
krs_file_year = atoi(fiwd);

krs_file_time = (struct tm *) malloc(sizeof(struct tm) );
if(! krs_file_time) return(0);

krs_file_time -> tm_sec = 0;
krs_file_time -> tm_min = 0;
krs_file_time -> tm_hour = 0;
/*krs_file_time -> tm_wday = 0;*/ /* to be calculated */
krs_file_time -> tm_mday = krs_file_day_of_month;
krs_file_time -> tm_mon = krs_file_month - 1;
krs_file_time -> tm_year = krs_file_year - 1900;
krs_file_time -> tm_isdst = -1;/* daylight savings time n.a. */
/* the following are not implemented (see /usr/include/time.h) */
/* krs_file_time -> tm_gmtoff = 0;*/
/* strcp(krs_file_time -> tm_zone, ""); */ /* time zone 3 letters */

old_data_time = mktime( (struct tm *)krs_file_time);
free(krs_file_time);

/* check if file already updated on this date */

if(new_data_time <= old_data_time)
	{
	sprintf(temp,\
	"update_krs_file(): file %s, has already been updated\n", ti);
	updateerrorfile(temp);
	fprintf(stdout, "%s", temp);
	fclose(krsf);
	fclose(krsfb);
	beep(2,3000);
	return(0);
	}

/* year new data from htm file */
sprintf(fiwd, "%d", htm_file_year);

/* write header to new file */
fprintf(krsf,"%s %s %d %s %4.2f %d %d %d %d %d %d %4.2f %4.2f %d ",\
fiwd, month_name[htm_file_month], htm_file_day_of_month, si,trend, af, pg, sp, xpos, ypos,\
lenfie, fmaxk, fmink, lenfr);

oldyear = 0;
fa = 0;/*reset text type flag*/
oldmonth[0] = 0; /* reset old month string */
xflag = 0;/*set for no ex div.*/
xcnt = 1; /*stock values (days) since last ex div.*/
special_field_appended_flag = 0;
while(1)/*copy rest of .bak file to .krs file*/
	{
	a = fscanf(krsfb,"%20s ", comstr);
	if(a == -1) break;/*EOF*/

	fprintf(krsf,"%s ",comstr);/*copy as string*/
	if(fa == -1)/*flag ind. text type month*/
		{
		strcpy(oldmonth, comstr);
		}
	if(fa == -2)/*comment*/
		{
		if(strstr(comstr,"X") != 0)/*ex div.*/
			{
			xflag=1;/*ex div. flag*/
			xcnt=1;/*reset the cnt*/
			}
		}
	if(fa == -3)/* flag indicates text type year */
		{
		oldyear = atoi(comstr);/*like 1996*/
		}
	fa = atof(comstr);/*fa=0 for non digits*/
	if(fa > 0)/*stock value read*/
		{
		if(xflag)xcnt++;
		/*days since last ex div.*/
		if(xcnt > 100)
			{
			xcnt=1;/*usally only once a year ex div*/
			xflag=0;/*no longer exdiv*/
			/*ready for next exdiv*/
			}
		fb=fa;/*remember last stock for repeat if closed*/
		}
	}/*end while copy all entries as string*/
			
/*append from here*/
/* the stock price */
if(data_type == NEW_DATA)/*new data was read*/
	{
	fprintf(krsf,"%.2f ", close_price);
	}
else /*negative values of units indicate special condition*/
	{
	if(data_type == CLOSED)/*text 'closed' was read */
					/*no trade, use prev value*/
		{
		fprintf(krsf,"%4.2f ", fb); /* repeat last value */
		}
	}

/* the special fields */
xdeflag = 0;/*reset last field is ex div entered flag*/
/* test for exdiv on this day */
if(data_type & EX_DIVIDENT)
	{
	if(! special_field_appended_flag)
		{
		if(!xflag)/*not already ex div*/
			{
			fprintf(krsf, "-2 X%d ", htm_file_day_of_month);
			xdeflag=1;/*set ex div entered as last field flag*/
			special_field_appended_flag = 1;
			}
		else /*was already exdiv*/
			{
			fprintf(stdout, \
"\nWARNING: X marker found, but already ex divident %d days ago, check.\n"\
			,xcnt);
			}
		}
	}/* end ex divident */

/* test if new month */
strcpy(comstr, month_name[htm_file_month]);/*month in alfa*/

if(strcmp(oldmonth, comstr) != 0)/*new month*/
	{
	if(!special_field_appended_flag)
		{
		/*
		if not just entered special field
		if special field was entered,
		print month next time
		in kra.exe:
		if 2 special fields after each other,
		txtptr[txtcnt] pointing to ex div
		field is overwritten with pointer
		to month field, so no X print in
		case 32
		*/
		fprintf(krsf,"-1 %s ",comstr);
		special_field_appended_flag = 1;
		}
	}/* end new month */
	
/* test if new year */
/* year insertion here using -3 below month, once in january */
if(htm_file_year != oldyear)/*year has changed*/
	{
	if(!special_field_appended_flag) /*not 2 special fields in row*/
		{
		fprintf(krsf, "-3 %d ", htm_file_year);/* append year field */
	  	oldyear = htm_file_year;
		special_field_appended_flag = 1;
		}
	}/* end new year */

fclose(krsf);/*close .tmp file (dest)*/
fclose(krsfb);/*close .krs file (source)*/

unlink(ta);/*erase any existing .bak file*/

i=rename(ti,ta);/*.krs into .bak*/
if(i == -1)
	{
	fprintf(stdout, "update_krs_file(): cannot rename file %s into %s\n",\
	ti,ta);
	return(0);
	}

i=rename(to,ti);/*.tmp into .krs*/
if(i == -1)
	{
	fprintf(stdout, "update_krs_file(): cannot rename file %s into %s\n",\
	to,ti);
	return(0);
	}

fprintf(stdout, "update_krs_file(): file %s has been updated\n", ti);

return(1);
}/* end function update_krs_file */


