/*
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#include "update_funds.h"

char *strsave(char *s) /* save char array s somewhere */
{
char *p;

p = malloc(strlen(s) +  1);
if(p) strcpy(p, s);
return(p);
}/* end function strsave */


char *strcasestr(char *str1, char *str2)
{
char *ptr1;
char *ptr2;
char *position;

position = str1;
while(1)
	{
	if(! *position) return(0);

	/* compare from position start */
	ptr1 = position;
	ptr2 = str2;
	while(1)
		{
		/* test for end string 2 first, that would be found */
		if(! *ptr2) return(position);

		/* test for end string 1 first, that would be failed */
		if(! *ptr1) break;

		if(tolower(*ptr1) == tolower(*ptr2) )
			{
			ptr1++;
			ptr2++;
			}
		else break;
		}/* end inner while */
	position++;
	}/* end outher while */
}/* end function strcasestr */


int readline(FILE *file, char *contents)
{
int a, c, i;

for(i = 0; i < READSIZE - 1; i++)
	{
	while(1)
		{
		c = getc(file);
		a = ferror(file);
		if(a)
			{
			perror("readline():");
			continue;
			}
		break;
		}
	if(feof(file) )
		{
		fclose(file);
		contents[i] = 0;/* EOF marker */
		return(EOF);
		}
	if(c == '\n')
		{
		contents[i] = 0;/* string termination */
		return(1);/* end of line */
		} 
	contents[i] = c;
	}
/*
mmm since we are here, the line must be quite long, possibly something
is wrong.
Since I do not always check for a return 0 in the use uf this function,
just to be safe, gona force a string termination.
This prevents stack overflow, and variables getting overwritten.
*/
contents[i] = 0;/* force string termination */
/*fclose(file);*/
if(debug_flag)
	{
	fprintf(stdout,\
	"readline(): line to long, returning 0 contents=%s\n", contents);
	}
return(0);
}/* end function readline */


int beep(int duration, int freq)
{
long i;
long lt;
int scale;

if(freq == 0)return(1);
if(duration == 0)return(1);
scale = 1193046 / freq;
outb(0xb6, 0x43);
outb(scale & 0xff, 0x42);
outb(scale >> 8, 0x42);
outb(inb(0x61) | 3, 0x61);/*set bit 0 and 1, timer 2 gate and lsp gate*/
	lt = duration * 6666666;/*depends on system speed*/
	for(i = 0; i < lt; i++)/*delay*/
		{
		;
		}
outb(inb(0x61) & ~3, 0x61);/* reset bit 0 and 1, timer 2 gate and lsp gate */
return(1);
}/* end function beep */


int useralarm()/*signal user*/
{
int i;
for(i=0; i < 5; i++)/*beep sequence 3 times*/
	{
	beep(1,3000);
	usleep(500000000);/* .5 second*/
	beep(1,4000);
	usleep(500000000);/* .5 second*/
	}
return(1);/*not same*/
}/* end function useralarm */


int ogettime(
char *weekday,
char *month,
int *monthday,
int *hour,
int *minute,
int *second,
int *year
)
{
unsigned long int I;
char *datestring;

time(&I);/*sets unsigned long int I to seconds since start 1970*/
datestring=ctime(&I);/*char *   converts I to character string date and time*/

sscanf(datestring,"%s %s %d %d:%d:%d %d",\
weekday,month,monthday,hour,minute,second,year);/*no &, already pointers*/

return(1);
}/* end function ogettime */


int get_path_name(char *request, char *reply)
/* reply is directory for requested type of file */
{
FILE *inifile;
int a, i;
char filename[40];
int flags[10];
char idfield[40], contentfield[40];
char database_dir[40];
char macro_files_dir[40];
char ir_remote_command_files_dir[40];
char acq_command_files_dir[40];
char stock_files_dir[40];
char error_files_dir[40];

if(debug_flag)
	{
	fprintf(stdout, "get_path_name(): arg request=%s\n", request);
	}

if(! request) return(0);

sprintf(filename, "/etc/krs/krs.ini");
inifile = fopen(filename, "r");
if(! inifile)
	{
	fprintf(stdout,\
	"get_path_name(): cannot open file %s for read\n", filename);
	return(0);

	strcpy(database_dir, "/teletext/");
	strcpy(macro_files_dir, "/etc/krs/");
	strcpy(ir_remote_command_files_dir, "/etc/krs/");
	strcpy(acq_command_files_dir, "/etc/krs/");
	strcpy(stock_files_dir, "/etc/krs/");
	strcpy(error_files_dir, "/etc/krs/");
	}
else /* reading from /etc/krs/krs.ini */
	{
	while(1)
		{
		a = fscanf(inifile, "%s %s", idfield, contentfield);
		if(a == EOF)break;
		if(strstr(idfield, "[database_files]") == idfield)
			{
			strcpy(database_dir, contentfield);
			flags[0] =1;
			}
		if(strstr(idfield, "[macro_files]") == idfield)
			{		
			strcpy(macro_files_dir, contentfield);
			flags[1] = 1;
			}
		if(strstr(idfield, "[ir_remote_command_files]") == idfield)
			{
			strcpy(ir_remote_command_files_dir, contentfield);
			flags[2] = 1;
			}
		if(strstr(idfield, "[acq_command_files]") == idfield)
			{
			strcpy(acq_command_files_dir, contentfield);
			flags[3] = 1;
			}
		if(strstr(idfield, "[stock_files]") == idfield)
			{
			strcpy(stock_files_dir, contentfield);
			flags[4] = 1;
			}
		if(strstr(idfield, "[error_files]") == idfield)
			{
			strcpy(error_files_dir, contentfield);
			flags[5] = 1;
			}		
		}/* end while read from file */
	for(i = 0; i < 6; i++)
		{
		if(flags[i] != 1)
			{
			fprintf(stdout, "get_path_name(): file format error field %d", i);
			}
		}
	fclose(inifile);
	}/* end reading from krs.ini */
/* reply */
if(strcmp("database_files_dir", request) == 0)\
	strcpy(reply, database_dir);
else if(strcmp("macro_files_dir", request) == 0)\
	strcpy(reply, macro_files_dir);
else if(strcmp("ir_remote_command_files_dir", request) == 0)\
	strcpy(reply, ir_remote_command_files_dir);
else if(strcmp("acq_command_files_dir", request) == 0)\
	strcpy(reply, acq_command_files_dir);
else if(strcmp("stock_files_dir", request) == 0)\
	strcpy(reply, stock_files_dir);
else if(strcmp("error_files_dir", request) == 0)\
	strcpy(reply, error_files_dir);
else if(inifile)/* error in krs.ini */
	{
	fprintf(stdout, "get_path_name(): Error in /etc/krs/krs.ini\n");
	if(strcmp("database_files_dir", request) == 0)strcpy(reply, "/teletext/");
	else strcpy(reply,"/etc/krs/");/* use if error in inifile */
	}

return(1);
}/* end function get_path_name */


int updateerrorfile(char *text)
{
FILE *errf;
char weekday[10];
char month[10];
int monthday, hour, minute, second, year;

if(debug_flag)
	{
	fprintf(stdout, "updateerrorfile(): arg text=%s\n", text);
	}

if(! text) return(0);

errf=fopen("errors.asc","a");/*create if not exist, update*/
if(!errf)
	{
	fprintf(stdout,\
	"updateerrorfile(): cannot open error file 'errors.asc' for append");
	return(0);
	}

ogettime(weekday, month, &monthday, &hour, &minute, &second, &year);

fprintf(errf,\
"\n%s %d %s %d %s",\
weekday, monthday, month, year, text);

fclose(errf);
return(1);
}/* end function updateerrorfile */


